@@ -20,7 +20,8 @@ G_DEFINE_TYPE(TangleCallFunctionAction, tangle_call_function_action, TANGLE_TYPE
 
 enum {
 	PROP_0,
-	PROP_FUNCTION
+	PROP_FUNCTION_NAME,
+	PROP_FUNCTION_POINTER
 };
 
 struct _TangleCallFunctionActionPrivate {
@@ -29,9 +30,14 @@ struct _TangleCallFunctionActionPrivate {
 	GModule* module;
 };
 
-TangleCallFunctionAction* tangle_call_function_action_new() {
+TangleAction* tangle_call_function_action_new(void) {
 
-	return TANGLE_CALL_FUNCTION_ACTION(g_object_new(TANGLE_TYPE_CALL_FUNCTION_ACTION, NULL));
+	return TANGLE_ACTION(g_object_new(TANGLE_TYPE_CALL_FUNCTION_ACTION, NULL));
+}
+
+TangleAction* tangle_call_function_action_new_with_function(TangleActionFunction function_pointer) {
+
+	return TANGLE_ACTION(g_object_new(TANGLE_TYPE_CALL_FUNCTION_ACTION, "function-pointer", function_pointer, NULL));
 }
 
 const gchar* tangle_call_function_action_get_function_name(TangleCallFunctionAction* action) {
@@ -40,10 +46,15 @@ const gchar* tangle_call_function_action_get_function_name(TangleCallFunctionAct
 }
 
 void tangle_call_function_action_set_function_name(TangleCallFunctionAction* action, const gchar* function_name) {
-	g_free(action->priv->function_name);
-	action->priv->function_name = g_strdup(function_name);
-	g_module_symbol(action->priv->module, function_name, (gpointer*)&action->priv->function_pointer);
-	g_object_notify(G_OBJECT(action), "function");
+	if (g_strcmp0(action->priv->function_name, function_name)) {
+		g_free(action->priv->function_name);
+		action->priv->function_name = g_strdup(function_name);
+		
+		g_module_symbol(action->priv->module, function_name, (gpointer*)&action->priv->function_pointer);
+
+		g_object_notify(G_OBJECT(action), "function-name");
+		g_object_notify(G_OBJECT(action), "function-pointer");
+	}
 }
 
 TangleActionFunction tangle_call_function_action_get_function_pointer(TangleCallFunctionAction* action) {
@@ -52,10 +63,15 @@ TangleActionFunction tangle_call_function_action_get_function_pointer(TangleCall
 }
 
 void tangle_call_function_action_set_function_pointer(TangleCallFunctionAction* action, TangleActionFunction function_pointer) {
-	action->priv->function_pointer = function_pointer;
-	g_free(action->priv->function_name);
-	action->priv->function_name = NULL;
-	g_object_notify(G_OBJECT(action), "function");
+	if (action->priv->function_pointer != function_pointer) {
+		action->priv->function_pointer = function_pointer;
+
+		g_free(action->priv->function_name);
+		action->priv->function_name = NULL;
+
+		g_object_notify(G_OBJECT(action), "function-name");
+		g_object_notify(G_OBJECT(action), "function-pointer");
+	}
 }
 
 static void tangle_call_function_action_execute(TangleAction* tangle_action, GObject* source, const gchar* trigger, TangleProperties* properties) {
@@ -73,9 +89,12 @@ static void tangle_call_function_action_set_property(GObject* object, guint prop
 	action = TANGLE_CALL_FUNCTION_ACTION(object);
 
 	switch (prop_id) {
-		case PROP_FUNCTION:
+		case PROP_FUNCTION_NAME:
 			tangle_call_function_action_set_function_name(action, g_value_get_string(value));
 			break;
+		case PROP_FUNCTION_POINTER:
+			tangle_call_function_action_set_function_pointer(action, g_value_get_pointer(value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 			break;
@@ -88,9 +107,12 @@ static void tangle_call_function_action_get_property(GObject* object, guint prop
 	action = TANGLE_CALL_FUNCTION_ACTION(object);
 
         switch (prop_id) {
-		case PROP_FUNCTION:
+		case PROP_FUNCTION_NAME:
 			g_value_set_string(value, action->priv->function_name);
 			break;
+		case PROP_FUNCTION_POINTER:
+			g_value_set_pointer(value, action->priv->function_pointer);
+			break;
 	        default:
 		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		        break;
@@ -98,6 +120,8 @@ static void tangle_call_function_action_get_property(GObject* object, guint prop
 }
 
 static void tangle_call_function_action_finalize(GObject* object) {
+	g_module_close(TANGLE_CALL_FUNCTION_ACTION(object)->priv->module);
+	
 	G_OBJECT_CLASS(tangle_call_function_action_parent_class)->finalize(object);
 }
 
@@ -117,20 +141,29 @@ static void tangle_call_function_action_class_init(TangleCallFunctionActionClass
 	action_class->execute = tangle_call_function_action_execute;
 
 	/**
-	 * TangleCallFunctionAction:function:
+	 * TangleCallFunctionAction:function-name:
 	 */
-	g_object_class_install_property(gobject_class, PROP_FUNCTION,
-	                                g_param_spec_string("function",
-	                                                    "Function",
+	g_object_class_install_property(gobject_class, PROP_FUNCTION_NAME,
+	                                g_param_spec_string("function-name",
+	                                                    "Function name",
 	                                                    "The name of the function that is called when the action is executed",
 	                                                    NULL,
 	                                                    G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleCallFunctionAction:function-pointer:
+	 */
+	g_object_class_install_property(gobject_class, PROP_FUNCTION_POINTER,
+	                                g_param_spec_pointer("function-pointer",
+	                                                     "Function pointer",
+	                                                     "A pointer to the function that is called when the action is executed",
+	                                                     G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
-	g_type_class_add_private (gobject_class, sizeof (TangleCallFunctionActionPrivate));
+	g_type_class_add_private(gobject_class, sizeof(TangleCallFunctionActionPrivate));
 }
 
 static void tangle_call_function_action_init(TangleCallFunctionAction* action) {
 	action->priv = G_TYPE_INSTANCE_GET_PRIVATE(action, TANGLE_TYPE_CALL_FUNCTION_ACTION, TangleCallFunctionActionPrivate);
+
 	action->priv->module = g_module_open(NULL, 0);
 	g_assert(action->priv->module != NULL);
 }
